const Main = imports.ui.main;
const Lang = imports.lang;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const St = imports.gi.St;
const Util = imports.misc.util;
const Gtk = imports.gi.Gtk;

const Gettext = imports.gettext.domain("gnome-shell-extensions-lock-screen");
const _ = Gettext.gettext;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = imports.misc.extensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const LockScreenIndicator = new Lang.Class({
	Name: "LockScreenIndicator",
	Extends: PanelMenu.Button,

	_init: function() {
		this.parent(0.0, "LockScreenIndicator");
				
		this._icon = new St.Icon({ icon_name: 'lock-screen-indicator-symbolic', style_class: 'system-status-icon' });
		this.actor.add_actor(this._icon);
	},

	buildMenu: function() {
		this.removeMenu();
		
		this._itemLockScreen = new PopupMenu.PopupMenuItem(_("Lock Screen"));
		this._itemLockScreen.connect("activate", Lang.bind(this, extension.lockScreen));
		this.menu.addMenuItem(this._itemLockScreen);
	
		this._itemDPMS = new PopupMenu.PopupMenuItem(_("Lock Screen / Turn off"));
		this._itemDPMS.connect("activate", Lang.bind(this, extension.lockScreenTurnOff));
		this.menu.addMenuItem(this._itemDPMS);
		
		this._itemSeparator = new PopupMenu.PopupSeparatorMenuItem();
		this.menu.addMenuItem(this._itemSeparator);

		this._itemSettings = new PopupMenu.PopupMenuItem(_("Settings"));
		this._itemSettings.connect("activate", Lang.bind(this, extension.launchSettings));
		this.menu.addMenuItem(this._itemSettings);
	},
	
	removeMenu: function() {
		if (!this.menu.isEmpty()) {
			this.menu.removeAll();
		}
	},
	
	setOnClick: function(enable, dpms) {
		if (enable) {
			this._onClickStateId = this.actor.connect("button-release-event", Lang.bind(this, function() { this._onClick(dpms); }));
		} else {
			if (this._onClickStateId) {
				this.actor.disconnect(this._onClickStateId);
			}
		}
	},
	
	_onClick: function(dpms) {
		if (dpms) {
			extension.lockScreenTurnOffMonitor();
		} else {			
			extension.lockScreen();
		}
	},
	
	hide: function() {
		this.actor.hide();
	},

	show: function() {
		this.actor.show();
	},

	destroy: function() {
		this.parent();
		
		this._icon = null;
		
		this._itemLockScreen.destroy();
		this._itemDPMS.destroy();
		this._itemSeparator.destroy();
		this._itemSettings.destroy();
		
		this._setOnClick(false, false);
		this._onClickStateId = null;
	}
});

const LockScreenUserMenu = new Lang.Class({
	Name: "LockScreenUserMenu",
	
	_init: function() {
		this._userMenu = Main.panel.statusArea.userMenu;
		let children = this._userMenu.menu._getMenuItems();
		let index = children.length;
		this._lockScreenSection = new PopupMenu.PopupMenuSection();	
		
		for (let i = index - 1; i >= 0; i--) {
			if (children[i] == this._userMenu._suspendOrPowerOffItem) {
				let item = new PopupMenu.PopupMenuItem(_("Lock Screen"));
				this._itemLockScreen = item;
				this._lockScreenSection.addMenuItem(item);
				item.connect("activate", Lang.bind(this, extension.lockScreen));
								
				item = new PopupMenu.PopupMenuItem(_("Lock Screen / Turn off"));
				this._itemDPMS = item;
				this._lockScreenSection.addMenuItem(item);
				item.connect("activate", Lang.bind(this, extension.lockScreenTurnOff));
				
				//this._lockScreenSection.addMenuItem(new PopupMenu.PopupSeparatorMenuItem);
								
				this._userMenu.menu.addMenuItem(this._lockScreenSection, i);
			}
		}
	},

	showLockScreen: function() {
		this._itemLockScreen.actor.show();
	},
	
	hideLockScreen: function() {
		this._itemLockScreen.actor.hide();
	},
	
	showLockScreenDPMS: function() {
		this._itemDPMS.actor.show();
	},
	
	hideLockScreenDPMS: function() {
		this._itemDPMS.actor.hide();
	},
	
	destroy: function() {
		let children = this._userMenu.menu._getMenuItems();
		let index = children.length;
		
		for (let i = index - 1; i >= 0; i--) {
			if (children[i] == this._lockScreenSection) {
				children[i].destroy();
			}
		}
	
		this._userMenu = null;
	}
});

const LockScreenExtension = new Lang.Class({
	Name: "LockScreenExtension",

	_init: function() {
		Convenience.initTranslations();
		
		this.settings = Convenience.getSettings();
		
		this._indicator = null;
		this._userMenu = null;
	},
	
	_createIndicator: function() {
		this._indicator = new LockScreenIndicator();			
		this._updateIndicatorMenu();

		Main.panel.addToStatusArea("lock-screen-indicator", this._indicator);
	},
	
	_updateIndicator: function() {
		if (this.settings.get_boolean("indicator")) {
			this._indicator.show();
		} else {
			this._indicator.hide();
		}
	},
	
	_updateIndicatorMenu: function() {
		this._indicator.setOnClick(false, false);
		
		if (this.settings.get_boolean("indicator-menu")) {
			this._indicator.buildMenu();
			this._indicator.setOnClick(false, false);
		} else {
			this._indicator.removeMenu();
			this._indicator.setOnClick(true, this.settings.get_boolean("indicator-dpms"));
		}
	},

	_updateUserMenu: function() {
		if (this.settings.get_boolean("user-menu-lock-screen")) {
			this._userMenu.showLockScreen();
		} else {
			this._userMenu.hideLockScreen();
		}
	
		if (this.settings.get_boolean("user-menu-dpms")) {
			this._userMenu.showLockScreenDPMS();
		} else {
			this._userMenu.hideLockScreenDPMS();
		}
	},
	
	lockScreen: function() {
		try {
			Util.spawn(["gnome-screensaver-command", "--lock"]);
		} catch(err) {
		}
	},

	lockScreenTurnOff: function() {
		try {
			Util.spawn(["gnome-screensaver-command", "--lock"]);
			Util.spawn(["xset", "dpms", "force", "off"]);
		} catch(err) {
		}
	},

	launchSettings: function() {
		try {
			Util.spawn(["gnome-shell-extension-prefs", "lock-screen@zardoz"]);
		} catch(err) {
		}
	},

	enable: function() {
		this._createIndicator();		
		this._updateIndicator();
		this._updateIndicatorMenu();
	
		this._userMenu = new LockScreenUserMenu();
		this._updateUserMenu();
	
		this._setting_changed = [
 			this.settings.connect("changed::indicator", Lang.bind(this, this._updateIndicator)),
 			this.settings.connect("changed::indicator-menu", Lang.bind(this, this._updateIndicatorMenu)),
 			this.settings.connect("changed::indicator-dpms", Lang.bind(this, this._updateIndicatorMenu)),
 			this.settings.connect("changed::user-menu-lock-screen", Lang.bind(this, this._updateUserMenu)),
 			this.settings.connect("changed::user-menu-dpms", Lang.bind(this, this._updateUserMenu))
		];
	},
	
	disable: function() {
		this._indicator.destroy();
		this._indicator = null;
		
		this._userMenu.destroy();
		this._userMenu = null;		
		
		for (let i in this._setting_changed) {
			this.settings.disconnect(this._setting_changed[i]);
		}
	
		this._setting_changed = null;
	}
});
	
let extension;

function init(extensionMeta) {
	let theme = imports.gi.Gtk.IconTheme.get_default();
	theme.append_search_path(extensionMeta.path + "/icons");
	
	return (extension = new LockScreenExtension());
}
